@@ -35,7 +35,7 @@
 #include "extern.h"
 #include "utility.h"
 
-const char *atacmds_c_cvsid="$Id: atacmds.c,v 1.153 2004/06/03 03:47:06 ballen4705 Exp $"
+const char *atacmds_c_cvsid="$Id: atacmds.c,v 1.161 2004/08/18 19:27:44 likewise Exp $"
 ATACMDS_H_CVSID CONFIG_H_CVSID EXTERN_H_CVSID INT64_H_CVSID UTILITY_H_CVSID;
 
 // to hold onto exit code for atexit routine
@@ -53,7 +53,7 @@ extern smartmonctrl *con;
 #define NOVAL_0                 0x0000
 #define NOVAL_1                 0xffff
 /* word 81: minor version number */
-#define MINOR_MAX 0x1e
+#define MINOR_MAX 0x22
 const char *minor_str[] = {                     /* word 81 value: */
   "Device does not report version",             /* 0x0000       */
   "ATA-1 X3T9.2 781D prior to revision 4",      /* 0x0001       */
@@ -85,7 +85,11 @@ const char *minor_str[] = {                     /* word 81 value: */
   "ATA/ATAPI-6 T13 1410D revision 2",           /* 0x001b       */
   "ATA/ATAPI-6 T13 1410D revision 1",           /* 0x001c       */
   "reserved",                                   /* 0x001d       */
-  "ATA/ATAPI-7 T13 1532D revision 0"            /* 0x001e       */
+  "ATA/ATAPI-7 T13 1532D revision 0",           /* 0x001e       */
+  "reserved",                                   /* 0x001f       */
+  "reserved",                                   /* 0x0020       */
+  "ATA/ATAPI-7 T13 1532D revision 4a",          /* 0x0021       */
+  "ATA/ATAPI-6 published, ANSI INCITS 361-2002" /* 0x0022       */
 };
 
 // NOTE ATA/ATAPI-4 REV 4 was the LAST revision where the device
@@ -126,7 +130,11 @@ const int actual_ver[] = {
   6,            /* 0x001b       WARNING:        */
   6,            /* 0x001c       WARNING:        */
   0,            /* 0x001d       WARNING:        */
-  7             /* 0x001e       WARNING:        */
+  7,            /* 0x001e       WARNING:        */
+  0,            /* 0x001f       WARNING:        */
+  0,            /* 0x0020       WARNING:        */
+  7,            /* 0x0021       WARNING:        */
+  6             /* 0x0022       WARNING:        */
 };
 
 // When you add additional items to this list, you should then:
@@ -595,11 +603,19 @@ int smartcommandhandler(int device, smart_command_set command, int select, char
   errno=0;
   
   // now execute the command
-  if (con->escalade)
-    retval=escalade_command_interface(device, con->escalade-1, command, select, data);
-  else
+  switch (con->controller_type) {
+  case CONTROLLER_3WARE_678K:
+  case CONTROLLER_3WARE_678K_CHAR:
+  case CONTROLLER_3WARE_9000_CHAR:
+    retval=escalade_command_interface(device, con->controller_port-1, con->controller_type, command, select, data);
+    break;
+  case CONTROLLER_MARVELL_SATA:
+    retval=marvell_command_interface(device, command, select, data);
+    break;
+  default:
     retval=ata_command_interface(device, command, select, data);
-  
+  }
+
   // If reporting is enabled, say what output was produced by the command
   if (con->reportataioctl){
     if (errno)
@@ -678,9 +694,14 @@ int ataReadHDIdentity (int device, struct ata_identify_device *buf){
     int i;
     
     // swap various capability words that are needed
-    swap2((char *)(rawshort+255)); 
+    for (i=0; i<33; i++)
+      swap2((char *)(buf->words047_079+i));
+    
     for (i=80; i<=87; i++)
       swap2((char *)(rawshort+i));
+    
+    for (i=0; i<168; i++)
+      swap2((char *)(buf->words088_255+i));
   }
   
   // If there is a checksum there, validate it
@@ -750,7 +771,7 @@ int ataVersionInfo (const char** description, struct ata_identify_device *drive,
   if (i==0)
     return 1;
   else
-    return i;;
+    return i;
 }
 
 // returns 1 if SMART supported, 0 if SMART unsupported, -1 if can't tell
@@ -1558,9 +1579,12 @@ int64_t ataPrintSmartAttribRawValue(char *out,
       out+=sprintf(out, "%"PRIu64, rawvalue);
     else {
       out+=sprintf(out, "%d", word[0]);
-      if (!(rawvalue==word[0]))
+      if (!(rawvalue==word[0])) {
+	int min=word[1]<word[2]?word[1]:word[2];
+	int max=word[1]>word[2]?word[1]:word[2];
         // The other bytes are in use. Try IBM's model
-        out+=sprintf(out, " (Lifetime Min/Max %d/%d)", word[1], word[2]);
+        out+=sprintf(out, " (Lifetime Min/Max %d/%d)", min, max);
+      }
     }
     break;
   default:
@@ -1744,7 +1768,7 @@ void ataPrintSmartAttribName(char *out, unsigned char id, unsigned char *definit
     break;
   case 206:
     // Fujitsu
-    name="Flying_Height_Measurement";
+    name="Flying_Height";
     break;
   case 207:
     // Maxtor
