@@ -26,7 +26,7 @@
 #include "utility.h" // includes <regex.h>
 #include "config.h"
 
-const char *knowndrives_c_cvsid="$Id: knowndrives.c,v 1.111 2004/07/05 07:41:52 ballen4705 Exp $"
+const char *knowndrives_c_cvsid="$Id: knowndrives.c,v 1.118 2004/08/21 15:20:39 pjwilliams Exp $"
 ATACMDS_H_CVSID ATAPRINT_H_CVSID CONFIG_H_CVSID EXTERN_H_CVSID INT64_H_CVSID KNOWNDRIVES_H_CVSID UTILITY_H_CVSID;
 
 #define MODEL_STRING_LENGTH                         40
@@ -145,7 +145,8 @@ const drivesettings knowndrives[] = {
     ".*",
     "IBM Deskstar 60GXP drives may need upgraded SMART firmware.\n"
     "Please see http://www.geocities.com/dtla_update/index.html#rel and\n"
-    "http://www-3.ibm.com/pc/support/site.wss/document.do?lndocid=MIGR-42215",
+    "http://www-3.ibm.com/pc/support/site.wss/document.do?lndocid=MIGR-42215 or\n"
+    "http://www-1.ibm.com/support/docview.wss?uid=psg1MIGR-42215",
     NULL, NULL, NULL
   },
   { // IBM Deskstar 40GV & 75GXP series (A5AA/A6AA firmware)
@@ -158,7 +159,8 @@ const drivesettings knowndrives[] = {
     ".*",
     "IBM Deskstar 40GV and 75GXP drives may need upgraded SMART firmware.\n"
     "Please see http://www.geocities.com/dtla_update/ and\n"
-    "http://www-3.ibm.com/pc/support/site.wss/document.do?lndocid=MIGR-42215",
+    "http://www-3.ibm.com/pc/support/site.wss/document.do?lndocid=MIGR-42215 or\n"
+    "http://www-1.ibm.com/support/docview.wss?uid=psg1MIGR-42215",
     NULL, NULL, NULL
   },
   { // ExcelStor J240
@@ -215,6 +217,13 @@ const drivesettings knowndrives[] = {
     vendoropts_Fujitsu_MHR2040AT,
     NULL, NULL
   },
+  { // Fujitsu MHR2020AT
+    "^FUJITSU MHR2020AT$",
+    ".*",
+    NULL,
+    vendoropts_9_seconds,
+    NULL, NULL
+  },
   { // Fujitsu MHSxxxxAT family
     "^FUJITSU MHS20[6432]0AT(  .)?$",
     ".*",
@@ -237,6 +246,13 @@ const drivesettings knowndrives[] = {
     vendoropts_9_seconds,
     NULL, NULL
   },
+  { // Fujitsu MHTxxxxAH family
+    "^FUJITSU MHT20[468]0AH$",
+    ".*",
+    NULL,
+    vendoropts_9_seconds,
+    NULL, NULL
+  },
   { // Samsung SV4012H (known firmware)
     "^SAMSUNG SV4012H$",
     "^RM100-08$",
@@ -364,6 +380,13 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
+  { // Maxtor DiamondMax Plus 5120 Ultra ATA 33 family
+    "^Maxtor 9(0512D2|0680D3|0750D3|0913D4|1024D4|1360D6|1536D6|1792D7|2048D8)$",
+    ".*",
+    NULL,
+    vendoropts_9_minutes,
+    NULL, NULL
+  },
   { // Maxtor DiamondMax Plus 6800 Ultra ATA 66 family
     "^Maxtor 9(2732U8|2390U7|2049U6|1707U5|1366U4|1024U3|0845U3|0683U2)$",
     ".*",
@@ -371,8 +394,8 @@ const drivesettings knowndrives[] = {
     vendoropts_9_minutes,
     NULL, NULL
   },
-  { // Maxtor 4D080H4
-    "^Maxtor (4D080H4|4G120J6)$",
+  { // Maxtor DiamondMax D540X-4D and Maxtor 4G120J6
+    "^Maxtor (4D0(20H1|40H2|60H3|80H4)|4G120J6)$",
     ".*",
     NULL,
     vendoropts_Maxtor_4D080H4,
@@ -519,11 +542,21 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL 
   },
+  { // IBM Travelstar 48GH, 30GN, and 15GN family
+    "^IC25(T048ATDA05|N0(30|20|15|12|10|07|06|05)ATDA04)-.$",
+    ".*",
+    NULL, NULL, NULL, NULL 
+  },
   { // IBM Travelstar 32GH, 30GT, and 20GN family
     "^IBM-DJSA-2(32|30|20|10|05)$",
     ".*",
     NULL, NULL, NULL, NULL 
   },
+  { // IBM Deskstar 37GP and 34GXP family
+    "^IBM-DPTA-3(5(375|300|225|150)|7(342|273|205|136))0$",
+    ".*",
+    NULL, NULL, NULL, NULL 
+  },
   { // IBM/Hitachi Travelstar 60GH and 40GN family
     "^IC25(T060ATC[SX]05|N0[4321]0ATC[SX]04)-.$",
     ".*",
@@ -535,10 +568,20 @@ const drivesettings knowndrives[] = {
     NULL, NULL, NULL, NULL 
   },
   { // Hitachi Travelstar 80GN family
-    "^IC25N0[23468]0ATMR04-.$",
+    "^(Hitachi )?IC25N0[23468]0ATMR04-.$",
     ".*",
     NULL, NULL, NULL, NULL
   },
+  { // Hitachi Travelstar 5K80 family
+    "^HTS5480[8642]0M9AT00$",
+    ".*",
+    NULL, NULL, NULL, NULL 
+  },
+  { // Hitachi Travelstar 7K60
+    "^HTS726060M9AT00$",
+    ".*",
+    NULL, NULL, NULL, NULL 
+  },
   { // IBM/Hitachi Deskstar 120GXP family
     "^IC35L((020|040|060|080|120)AVVA|0[24]0AVVN)07-[01]$",
     ".*",
@@ -589,8 +632,8 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
-  { // TOSHIBA MK2017GAP
-    "^TOSHIBA MK2017GAP$",
+  { // TOSHIBA MK2016GAP, MK2017GAP, MK2018GAP, MK2018GAS, MK2023GAS
+    "^TOSHIBA MK20(1[678]GAP|(18|23)GAS)$",
     ".*",
     NULL, NULL, NULL, NULL
   },
@@ -599,6 +642,11 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
+  { // TOSHIBA MK3017GAP
+    "^TOSHIBA MK3017GAP$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
   { // Seagate Medalist 8641 family
     "^ST3(2110|3221|4312|6531|8641)A$",
     ".*",
@@ -624,6 +672,11 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
+  { // Seagate U10 family
+    "^ST3(20423|15323|10212)A$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
   { // Seagate Barracuda ATA II family
     "^ST3(3063|2042|1532|1021)0A$",
     ".*",
@@ -690,7 +743,7 @@ const drivesettings knowndrives[] = {
    * is understood exactly how Attribute 9 should be interpreted.
    * UPDATE: this is probably explained by the WD firmware bug described in the
    * smartmontools FAQ */
-    "^WDC WD(64|84|102|136|205|307)AA(-.*)?$",
+    "^WDC WD(64|84|102|136|205|272|307)AA(-.*)?$",
     ".*",
     NULL, NULL, NULL, NULL
   },
@@ -715,7 +768,7 @@ const drivesettings knowndrives[] = {
    * is understood exactly how Attribute 9 should be interpreted.
    * UPDATE: this is probably explained by the WD firmware bug described in the
    * smartmontools FAQ */
-    "^WDC WD(4|6|8|10|12|16|18|20|25)00JB-.*$",
+    "^WDC WD((4|6|8|10|12|16|18|20|25)00JB|(12|20|25)00PB)-.*$",
     ".*",
     NULL, NULL, NULL, NULL
   },
@@ -759,8 +812,8 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
-  { // QUANTUM FIREBALLP AS10.2 and AS40.0
-    "^QUANTUM FIREBALLP AS(10.2|40.0)$",
+  { // QUANTUM FIREBALLP AS10.2, AS20.5, and AS40.0
+    "^QUANTUM FIREBALLP AS(10.2|20.5|40.0)$",
     ".*",
     NULL, NULL, NULL, NULL
   },
@@ -769,6 +822,16 @@ const drivesettings knowndrives[] = {
     ".*",
     NULL, NULL, NULL, NULL
   },
+  { // QUANTUM FIREBALL ST3.2A
+    "^QUANTUM FIREBALL ST3.2A$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
+  { // QUANTUM FIREBALL EX3.2A
+    "^QUANTUM FIREBALL EX3.2A$",
+    ".*",
+    NULL, NULL, NULL, NULL
+  },
   /*------------------------------------------------------------
    *  End of table.  Do not add entries below this marker.
    *------------------------------------------------------------ */
