@@ -33,11 +33,13 @@
 // This is to include whatever prototypes you define in os_solaris.h
 #include "os_solaris.h"
 
+#define ARGUSED(x) ((void)(x))
+
 extern long long bytes;
 
-static const char *filenameandversion="$Id: os_solaris.c,v 1.17 2004/03/13 02:32:52 ballen4705 Exp $";
+static const char *filenameandversion="$Id: os_solaris.c,v 1.21 2004/08/28 23:23:46 card_captor Exp $";
 
-const char *os_XXXX_c_cvsid="$Id: os_solaris.c,v 1.17 2004/03/13 02:32:52 ballen4705 Exp $" \
+const char *os_XXXX_c_cvsid="$Id: os_solaris.c,v 1.21 2004/08/28 23:23:46 card_captor Exp $" \
 ATACMDS_H_CVSID CONFIG_H_CVSID OS_XXXX_H_CVSID SCSICMDS_H_CVSID UTILITY_H_CVSID;
 
 // The printwarning() function warns about unimplemented functions
@@ -141,11 +143,11 @@ isatadev(const char *path)
 // tries to guess device type given the name (a path)
 int guess_device_type (const char* dev_name) {
   if (isscsidev(dev_name))
-    return GUESS_DEVTYPE_SCSI;
+    return CONTROLLER_SCSI;
   else if (isatadev(dev_name))
-    return GUESS_DEVTYPE_ATA;
+    return CONTROLLER_ATA;
   else
-    return GUESS_DEVTYPE_DONT_KNOW;
+    return CONTROLLER_UNKNOWN;
 }
 
 struct pathlist {
@@ -278,6 +280,11 @@ static void swap_sector(void *p)
 }
 
 // Interface to ATA devices.  See os_linux.c
+int marvell_command_interface(int fd, smart_command_set command, int select, char *data){
+    ARGUSED(fd); ARGUSED(command); ARGUSED(select); ARGUSED(data);
+    return -1;
+}
+
 int ata_command_interface(int fd, smart_command_set command, int select, char *data){
 #if defined(__sparc)
     int err;
@@ -336,9 +343,9 @@ int ata_command_interface(int fd, smart_command_set command, int select, char *d
 }
 
 // Interface to ATA devices behind 3ware escalade RAID controller cards.  See os_linux.c
-int escalade_command_interface(int fd, int disknum, smart_command_set command, int select, char *data){
+int escalade_command_interface(int fd, int disknum, int escalade_type, smart_command_set command, int select, char *data){
   // avoid gcc warnings//
-  fd=disknum=command=select=0;
+  fd=disknum=escalade_type=command=select=0;
   data=NULL;
 
   if (printwarning(1))
