@@ -39,7 +39,7 @@ extern int64_t bytes; // malloc() byte count
 #define ARGUSED(x) ((void)(x))
 
 // Needed by '-V' option (CVS versioning) of smartd/smartctl
-const char *os_XXXX_c_cvsid="$Id: os_win32.c,v 1.14 2004/06/24 12:28:39 chrfranke Exp $"
+const char *os_XXXX_c_cvsid="$Id: os_win32.c,v 1.17 2004/08/18 19:27:44 likewise Exp $"
 ATACMDS_H_CVSID CONFIG_H_CVSID EXTERN_H_CVSID INT64_H_CVSID SCSICMDS_H_CVSID UTILITY_H_CVSID;
 
 
@@ -74,10 +74,10 @@ int guess_device_type (const char * dev_name)
 {
 	dev_name = skipdev(dev_name);
 	if (!strncmp(dev_name, "hd", 2))
-		return GUESS_DEVTYPE_ATA;
+		return CONTROLLER_ATA;
 	if (!strncmp(dev_name, "scsi", 4))
-		return GUESS_DEVTYPE_SCSI;
-	return GUESS_DEVTYPE_DONT_KNOW;
+		return CONTROLLER_SCSI;
+	return CONTROLLER_UNKNOWN;
 }
 
 
@@ -658,6 +658,9 @@ static unsigned ata_scan()
 /////////////////////////////////////////////////////////////////////////////
 
 // Interface to ATA devices.  See os_linux.c
+int marvell_command_interface(int fd, smart_command_set command, int select, char * data)
+{ return -1; }
+
 int ata_command_interface(int fd, smart_command_set command, int select, char * data)
 {
 	IDEREGS regs;
@@ -781,10 +784,10 @@ int ata_command_interface(int fd, smart_command_set command, int select, char *
 
 
 // Interface to ATA devices behind 3ware escalade RAID controller cards.  See os_linux.c
-int escalade_command_interface(int fd, int disknum, smart_command_set command, int select, char *data)
+int escalade_command_interface(int fd, int disknum, int escalade_type, smart_command_set command, int select, char *data)
 {
 	static int warned = 0;
-	ARGUSED(fd); ARGUSED(disknum); ARGUSED(command); ARGUSED(select); ARGUSED(data);
+	ARGUSED(fd); ARGUSED(disknum); ARGUSED(escalade_type); ARGUSED(command); ARGUSED(select); ARGUSED(data);
 	if (!warned) {
 		  pout(
 		"#######################################################################\n"
