@@ -47,7 +47,7 @@
 #include "scsicmds.h"
 #include "utility.h"
 
-const char *scsicmds_c_cvsid="$Id: scsicmds.c,v 1.75 2004/04/30 06:15:49 dpgilbert Exp $"
+const char *scsicmds_c_cvsid="$Id: scsicmds.c,v 1.78 2004/09/05 13:53:14 dpgilbert Exp $"
 CONFIG_H_CVSID EXTERN_H_CVSID INT64_H_CVSID SCSICMDS_H_CVSID UTILITY_H_CVSID;
 
 /* for passing global control variables */
@@ -176,7 +176,8 @@ static int scsiSimpleSenseFilter(const struct scsi_sense_disect * sinfo)
             return SIMPLE_ERR_BAD_FIELD;
         else if (SCSI_ASC_UNKNOWN_PARAM == sinfo->asc)
             return SIMPLE_ERR_BAD_PARAM;
-    }
+    } else if (SCSI_SK_UNIT_ATTENTION == sinfo->sense_key)
+        return SIMPLE_ERR_TRY_AGAIN;
     return SIMPLE_NO_ERROR;
 }
 
@@ -201,6 +202,8 @@ const char * scsiErrString(int scsiErr)
             return "no medium present";
         case SIMPLE_ERR_BECOMING_READY: 
             return "device will be ready soon";
+        case SIMPLE_ERR_TRY_AGAIN: 
+            return "unit attention reported, try again";
         default:
             return "unknown error";
     }
@@ -331,18 +334,25 @@ int scsiModeSense(int device, int pagenum, int pc, UINT8 *pBuf, int bufLen)
     io_hdr.timeout = SCSI_TIMEOUT_DEFAULT;
 
     status = do_scsi_cmnd_io(device, &io_hdr, con->reportscsiioctl);
-    if (0 == status) {
+    if (0 != status)
+	return status;
+    scsi_do_sense_disect(&io_hdr, &sinfo);
+    status = scsiSimpleSenseFilter(&sinfo);
+    if (SIMPLE_ERR_TRY_AGAIN == status) {
+        status = do_scsi_cmnd_io(device, &io_hdr, con->reportscsiioctl);
+        if (0 != status)
+	    return status;
         scsi_do_sense_disect(&io_hdr, &sinfo);
         status = scsiSimpleSenseFilter(&sinfo);
-        if ((0 == status) && (ALL_MODE_PAGES != pagenum)) {
-            int offset;
+    }
+    if ((0 == status) && (ALL_MODE_PAGES != pagenum)) {
+        int offset;
 
-            offset = scsiModePageOffset(pBuf, bufLen, 0);
-            if (offset < 0)
-                return SIMPLE_ERR_BAD_RESP;
-            else if (pagenum != (pBuf[offset] & 0x3f))
-                return SIMPLE_ERR_BAD_RESP;
-        }
+        offset = scsiModePageOffset(pBuf, bufLen, 0);
+        if (offset < 0)
+            return SIMPLE_ERR_BAD_RESP;
+        else if (pagenum != (pBuf[offset] & 0x3f))
+            return SIMPLE_ERR_BAD_RESP;
     }
     return status;
 }
@@ -420,18 +430,25 @@ int scsiModeSense10(int device, int pagenum, int pc, UINT8 *pBuf, int bufLen)
     io_hdr.timeout = SCSI_TIMEOUT_DEFAULT;
 
     status = do_scsi_cmnd_io(device, &io_hdr, con->reportscsiioctl);
-    if (0 == status) {
+    if (0 != status)
+	return status;
+    scsi_do_sense_disect(&io_hdr, &sinfo);
+    status = scsiSimpleSenseFilter(&sinfo);
+    if (SIMPLE_ERR_TRY_AGAIN == status) {
+        status = do_scsi_cmnd_io(device, &io_hdr, con->reportscsiioctl);
+        if (0 != status)
+	    return status;
         scsi_do_sense_disect(&io_hdr, &sinfo);
         status = scsiSimpleSenseFilter(&sinfo);
-        if ((0 == status) && (ALL_MODE_PAGES != pagenum)) {
-            int offset;
+    }
+    if ((0 == status) && (ALL_MODE_PAGES != pagenum)) {
+        int offset;
 
-            offset = scsiModePageOffset(pBuf, bufLen, 1);
-            if (offset < 0)
-                return SIMPLE_ERR_BAD_RESP;
-            else if (pagenum != (pBuf[offset] & 0x3f))
-                return SIMPLE_ERR_BAD_RESP;
-        }
+        offset = scsiModePageOffset(pBuf, bufLen, 1);
+        if (offset < 0)
+            return SIMPLE_ERR_BAD_RESP;
+        else if (pagenum != (pBuf[offset] & 0x3f))
+            return SIMPLE_ERR_BAD_RESP;
     }
     return status;
 }
@@ -706,14 +723,15 @@ int scsiTestUnitReady(int device)
     status = _testunitready(device, &sinfo);
     if (0 != status)
         return status;
-
-    if (SCSI_SK_UNIT_ATTENTION == sinfo.sense_key) {
+    status = scsiSimpleSenseFilter(&sinfo);
+    if (SIMPLE_ERR_TRY_AGAIN == status) {
         /* power on reset, media changed, ok ... try again */
         status = _testunitready(device, &sinfo);        
         if (0 != status)
             return status;
+        status = scsiSimpleSenseFilter(&sinfo);
     }
-    return scsiSimpleSenseFilter(&sinfo);
+    return status;
 }
 
 /* Offset into mode sense (6 or 10 byte) response that actual mode page
@@ -1693,6 +1711,36 @@ void scsiDecodeNonMediumErrPage(unsigned char *resp,
                     nmep->counterPC0 |= xp[j];
                 }
                 break;
+            case 0x8009: 
+                nmep->gotTFE_H = 1;
+                k = pl - 4;
+                xp = ucp + 4;
+                if (k > szof) {
+                    xp += (k - szof);
+                    k = szof;
+                }
+                nmep->counterTFE_H = 0;
+                for (j = 0; j < k; ++j) {
+                    if (j > 0)
+                        nmep->counterTFE_H <<= 8;
+                    nmep->counterTFE_H |= xp[j];
+                }
+                break;
+            case 0x8015: 
+                nmep->gotPE_H = 1;
+                k = pl - 4;
+                xp = ucp + 4;
+                if (k > szof) {
+                    xp += (k - szof);
+                    k = szof;
+                }
+                nmep->counterPE_H = 0;
+                for (j = 0; j < k; ++j) {
+                    if (j > 0)
+                        nmep->counterPE_H <<= 8;
+                    nmep->counterPE_H |= xp[j];
+                }
+                break;
         default: 
                 nmep->gotExtraPC = 1;
                 break;
@@ -1922,3 +1970,38 @@ int scsiFetchTransportProtocol(int device, int modese_len)
     return -EINVAL;
 }
 
+
+/* This is Linux specific code to look for the libata ATA-SCSI
+   simulator in the vendor device identification page.
+   Returns 1 if found else 0. */
+int isLinuxLibAta(unsigned char * vpd_di_buff, int len)
+{
+    int k, id_len, c_set, assoc, id_type, i_len;
+    unsigned char * ucp;
+    unsigned char * ip;
+
+    if (len < 4) {
+        /* Device identification VPD page length too short */
+        return 0;
+    }
+    len -= 4;
+    ucp = vpd_di_buff + 4;
+    for (k = 0; k < len; k += id_len, ucp += id_len) {
+        i_len = ucp[3];
+        id_len = i_len + 4;
+        if ((k + id_len) > len) {
+            /* short descriptor, badly formed */
+            return 0;
+        }
+        ip = ucp + 4;
+        c_set = (ucp[0] & 0xf);
+        assoc = ((ucp[1] >> 4) & 0x3);
+        id_type = (ucp[1] & 0xf);
+        if ((0 == id_type) && (2 == c_set) && (0 == assoc) &&
+            (0 == strncmp((const char *)ip,
+                          "Linux ATA-SCSI simulator", i_len))) {
+            return 1;
+        }
+    }
+    return 0;
+}
