@@ -32,7 +32,7 @@
 #ifndef SCSICMDS_H_
 #define SCSICMDS_H_
 
-#define SCSICMDS_H_CVSID "$Id: scsicmds.h,v 1.48 2004/03/20 08:52:40 dpgilbert Exp $\n"
+#define SCSICMDS_H_CVSID "$Id: scsicmds.h,v 1.52 2004/09/05 13:52:05 dpgilbert Exp $\n"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -135,6 +135,10 @@ struct scsiNonMediumError {
     UINT8 gotPC0;
     UINT8 gotExtraPC;
     uint64_t counterPC0;
+    UINT8 gotTFE_H;
+    uint64_t counterTFE_H; /* Track following errors [Hitachi] */
+    UINT8 gotPE_H;
+    uint64_t counterPE_H;  /* Positioning errors [Hitachi] */
 };
 
 /* SCSI Peripheral types (of interest) */
@@ -229,6 +233,7 @@ Documentation, see http://www.storage.ibm.com/techsup/hddtech/prodspecs.htm */
 #define SIMPLE_ERR_BAD_RESP             5       /* response fails sanity */
 #define SIMPLE_ERR_NO_MEDIUM            6       /* no medium present */
 #define SIMPLE_ERR_BECOMING_READY       7       /* device will be ready soon */
+#define SIMPLE_ERR_TRY_AGAIN            8       /* some warning, try again */
 
 
 /* defines for functioncode parameter in SENDDIAGNOSTIC function */
@@ -325,7 +330,19 @@ const char * scsiTapeAlertsChangerDevice(unsigned short code);
 const char * scsi_get_opcode_name(UINT8 opcode);
 void dStrHex(const char* str, int len, int no_ascii);
 
+/* SCSI command transmission interface function declaration. Its
+ * definition is target OS specific (see os_<OS>.c file).
+ * Returns 0 if SCSI command successfully launched and response
+ * received. Even when 0 is returned the caller should check
+ * scsi_cmnd_io::scsi_status for SCSI defined errors and warnings
+ * (e.g. CHECK CONDITION). If the SCSI command could not be issued
+ * (e.g. device not present or not a SCSI device) or some other problem
+ * arises (e.g. timeout) then returns a negative errno value. */
 int do_scsi_cmnd_io(int dev_fd, struct scsi_cmnd_io * iop, int report);
 
+
+/* This is Linux specific and will be generalized later. */
+int isLinuxLibAta(unsigned char * vpd_di_buff, int len);
+
 #endif
 
