@@ -41,7 +41,7 @@
 
 #define GBUF_SIZE 65535
 
-const char* scsiprint_c_cvsid="$Id: scsiprint.c,v 1.78 2004/04/30 06:17:52 dpgilbert Exp $"
+const char* scsiprint_c_cvsid="$Id: scsiprint.c,v 1.89 2004/09/06 03:41:13 dpgilbert Exp $"
 CONFIG_H_CVSID EXTERN_H_CVSID INT64_H_CVSID SCSICMDS_H_CVSID SCSIPRINT_H_CVSID SMARTCTL_H_CVSID UTILITY_H_CVSID;
 
 // control block which points to external global control variables
@@ -52,6 +52,7 @@ extern int exitstatus;
 
 UINT8 gBuf[GBUF_SIZE];
 #define LOG_RESP_LEN 252
+#define LOG_RESP_LONG_LEN 8192
 #define LOG_RESP_TAPE_ALERT_LEN 0x144
 
 /* Log pages supported */
@@ -340,7 +341,7 @@ static void scsiPrintSeagateFactoryLPage(int device)
     }
     if (gBuf[0] != SEAGATE_FACTORY_LPAGE) {
         PRINT_ON(con);
-        pout("Seagate Factory Log Sense Failed, page mismatch\n");
+        pout("Seagate/Hitachi Factory Log Sense Failed, page mismatch\n");
         PRINT_OFF(con);
         return;
     }
@@ -356,8 +357,8 @@ static void scsiPrintSeagateFactoryLPage(int device)
         default: 
             if (con->reportscsiioctl > 0) {
                 PRINT_ON(con);
-                pout("\nVendor (Seagate) factory lpage has unexpected "
-                     "parameter, skip\n");
+                pout("\nVendor (Seagate/Hitachi) factory lpage has unexpected"
+                     " parameter, skip\n");
                 PRINT_OFF(con);
             }
             return;
@@ -365,7 +366,7 @@ static void scsiPrintSeagateFactoryLPage(int device)
         num -= pl;
         ucp += pl;
     }
-    pout("Vendor (Seagate) factory information\n");
+    pout("Vendor (Seagate/Hitachi) factory information\n");
     num = len - 4;
     ucp = &gBuf[0] + 4;
     while (num > 3) {
@@ -375,7 +376,7 @@ static void scsiPrintSeagateFactoryLPage(int device)
         case 0: pout("  number of hours powered up"); break;
         case 8: pout("  number of minutes until next internal SMART test");
             break;
-        default: pout("  Unknown Seagate parameter code [0x%x]", pc); break;
+        default: pout("  Unknown parameter code [0x%x]", pc); break;
         }
         k = pl - 4;
         xp = ucp + 4;
@@ -431,11 +432,11 @@ static void scsiPrintErrorCounterLog(int device)
     }
     if (found[0] || found[1] || found[2]) {
         pout("\nError counter log:\n");
-        pout("          Errors Corrected    Total      Total   "
+        pout("           Errors Corrected by           Total   "
              "Correction     Gigabytes    Total\n");
-        pout("              delay:       [rereads/    errors   "
+        pout("               EEC          rereads/    errors   "
              "algorithm      processed    uncorrected\n");
-        pout("            minor | major  rewrites]  corrected  "
+        pout("           fast | delayed   rewrites  corrected  "
              "invocations   [10^9 bytes]  errors\n");
         for (k = 0; k < 3; ++k) {
             if (! found[k])
@@ -449,13 +450,50 @@ static void scsiPrintErrorCounterLog(int device)
         }
     }
     else 
-        pout("\nDevice does not support Error Counter logging\n");
+        pout("\nError Counter logging not supported\n");
     if (0 == scsiLogSense(device, NON_MEDIUM_ERROR_LPAGE, gBuf, 
                           LOG_RESP_LEN, 0)) {
         scsiDecodeNonMediumErrPage(gBuf, &nme);
         if (nme.gotPC0)
             pout("\nNon-medium error count: %8"PRIu64"\n", nme.counterPC0);
+        if (nme.gotTFE_H)
+            pout("Track following error count [Hitachi]: %8"PRIu64"\n",
+                 nme.counterTFE_H);
+        if (nme.gotPE_H)
+            pout("Positioning error count [Hitachi]: %8"PRIu64"\n",
+                 nme.counterPE_H);
     }
+    if (0 == scsiLogSense(device, LAST_N_ERROR_LPAGE, gBuf, 
+                          LOG_RESP_LONG_LEN, 0)) {
+        unsigned char * ucp;
+        int num, k, pc, pl;
+
+        num = (gBuf[2] << 8) + gBuf[3];
+        num = (num < LOG_RESP_LONG_LEN) ? num : LOG_RESP_LONG_LEN;
+        ucp = gBuf + 4;
+        if (num < 4)
+            pout("\nNo error events logged\n");
+        else {
+            pout("Last n error events log page\n");
+            for (k = num; k > 0; k -= pl, ucp += pl) {
+                if (k < 3) {
+                    pout("  <<short Last n error events log page>>\n");
+                    break;
+                }
+                pl = ucp[3] + 4;
+                pc = (ucp[0] << 8) + ucp[1];
+                pout("  Error event %d:\n", pc);
+                if (ucp[2] & 0x2) {
+                    pout("    [binary]:\n");
+                    dStrHex((const char *)ucp + 4, pl - 4, 1);
+                } else
+                    pout("    %.*s\n", pl - 4, (const char *)(ucp + 4));
+                num -= pl;
+                ucp += pl;
+            }
+        }
+    } else
+        pout("\nError Events logging not supported\n");
 }
 
 const char * self_test_code[] = {
@@ -636,8 +674,8 @@ static const char * transport_proto_arr[] = {
         "0xe",
         "0xf"
 };
- 
-/* Returns 0 on success */
+
+/* Returns 0 on success, 1 on general error and 2 for early, clean exit */
 static int scsiGetDriveInfo(int device, UINT8 * peripheral_type, int all)
 {
     char manufacturer[9];
@@ -645,19 +683,30 @@ static int scsiGetDriveInfo(int device, UINT8 * peripheral_type, int all)
     char revision[5];
     char timedatetz[DATEANDEPOCHLEN];
     struct scsi_iec_mode_page iec;
-    int err, iec_err, len, val;
+    int err, iec_err, len, req_len, avail_len, val;
     int is_tape = 0;
     int peri_dt = 0;
     int returnval=0;
         
-    memset(gBuf, 0, 36);
-    if ((err = scsiStdInquiry(device, gBuf, 36))) {
+    memset(gBuf, 0, 96);
+    req_len = 36;
+    if ((err = scsiStdInquiry(device, gBuf, req_len))) {
         PRINT_ON(con);
-        pout("Standard Inquiry failed [%s]\n", scsiErrString(err));
+        pout("Standard Inquiry (36 bytes) failed [%s]\n", scsiErrString(err));
+        pout("Retrying with a 64 byte Standard Inquiry\n");
         PRINT_OFF(con);
-        return 1;
+        /* Marvell controllers fail on a 36 bytes StdInquiry, but 64 suffices */
+        req_len = 64;
+        if ((err = scsiStdInquiry(device, gBuf, req_len))) {
+            PRINT_ON(con);
+            pout("Standard Inquiry (64 bytes) failed [%s]\n",
+                 scsiErrString(err));
+            PRINT_OFF(con);
+            return 1;
+        }
     }
-    len = gBuf[4] + 5;
+    avail_len = gBuf[4] + 5;
+    len = (avail_len < req_len) ? avail_len : req_len;
     peri_dt = gBuf[0] & 0x1f;
     if (peripheral_type)
         *peripheral_type = peri_dt;
@@ -680,6 +729,36 @@ static int scsiGetDriveInfo(int device, UINT8 * peripheral_type, int all)
     strncpy(revision, (char *)&gBuf[32], 4);
     pout("Device: %s %s Version: %s\n", manufacturer, product, revision);
 
+    if (0 == strncmp(manufacturer, "3ware", 5)) {
+        pout("please try '-d 3ware,N'\n");
+        return 2;
+    } else if ((len >= 42) && (0 == strncmp(&gBuf[36], "MVSATA", 6))) {
+        pout("please try '-d marvell'\n");
+        return 2;
+    } else if ((avail_len >= 96) && (0 == strncmp(manufacturer, "ATA", 3))) {
+        /* <<<< This is Linux specific code to detect SATA disks using a
+                SCSI-ATA command translation layer. This may be generalized
+                later when the t10.org SAT project matures. >>>> */
+        req_len = 96;
+        memset(gBuf, 0, req_len);
+        if ((err = scsiInquiryVpd(device, 0x83, gBuf, req_len))) {
+            PRINT_ON(con);
+            pout("Inquiry for VPD page 0x83 [device id] failed [%s]\n",
+                  scsiErrString(err));
+            PRINT_OFF(con);
+            return 1;
+        }
+        avail_len = ((gBuf[2] << 8) + gBuf[3]) + 4;
+        len = (avail_len < req_len) ? avail_len : req_len;
+        if (isLinuxLibAta(gBuf, len)) {
+            pout("\nSATA disks accessed via libata are not currently "
+                 "supported by\nsmartmontools. When libata is given "
+                 "an ATA pass-thru ioctl() then an\nadditional '-d libata'"
+                 " device type will be added to smartmontools.\n");
+            return 2;
+        }
+    }
+
     /* Do this here to try and detect badly conforming devices (some USB
        keys) that will lock up on a InquiryVpd or log sense or ... */
     if ((iec_err = scsiFetchIECmpage(device, &iec, modese_len))) {
@@ -732,7 +811,10 @@ static int scsiGetDriveInfo(int device, UINT8 * peripheral_type, int all)
     if ((err = scsiTestUnitReady(device))) {
         if (SIMPLE_ERR_NOT_READY == err) {
             PRINT_ON(con);
-            pout("device is NOT READY (e.g. spun down, busy)\n");
+            if (!is_tape)
+                pout("device is NOT READY (e.g. spun down, busy)\n");
+            else
+                pout("device is NOT READY (e.g. no tape)\n");
             PRINT_OFF(con);
          } else if (SIMPLE_ERR_NO_MEDIUM == err) {
             PRINT_ON(con);
@@ -871,8 +953,12 @@ int scsiPrintMain(int fd)
     int returnval = 0;
     int res, durationSec;
 
-    if (scsiGetDriveInfo(fd, &peripheral_type, con->driveinfo)) {
-        failuretest(MANDATORY_CMD, returnval |= FAILID);
+    res = scsiGetDriveInfo(fd, &peripheral_type, con->driveinfo);
+    if (res) {
+        if (2 == res)
+            return 0;
+        else
+            failuretest(MANDATORY_CMD, returnval |= FAILID);
     }
 
     if (con->smartenable) {
