@@ -31,6 +31,9 @@
 #ifdef HAVE_GETOPT_LONG
 #include <getopt.h>
 #endif
+#if defined(__FreeBSD_version) && (__FreeBSD_version < 500000)
+#include <unistd.h>
+#endif
 #include "atacmds.h"
 #include "ataprint.h"
 #include "extern.h"
@@ -48,7 +51,7 @@ extern const char *os_solaris_ata_s_cvsid;
 extern const char *int64_vc6_c_cvsid;
 #endif
 extern const char *atacmdnames_c_cvsid, *atacmds_c_cvsid, *ataprint_c_cvsid, *knowndrives_c_cvsid, *os_XXXX_c_cvsid, *scsicmds_c_cvsid, *scsiprint_c_cvsid, *utility_c_cvsid;
-const char* smartctl_c_cvsid="$Id: smartctl.c,v 1.125 2004/04/30 06:23:59 dpgilbert Exp $"
+const char* smartctl_c_cvsid="$Id: smartctl.c,v 1.135 2004/08/18 19:27:45 likewise Exp $"
 ATACMDS_H_CVSID ATAPRINT_H_CVSID CONFIG_H_CVSID EXTERN_H_CVSID INT64_H_CVSID KNOWNDRIVES_H_CVSID SCSICMDS_H_CVSID SCSIPRINT_H_CVSID SMARTCTL_H_CVSID UTILITY_H_CVSID;
 
 // This is a block containing all the "control variables".  We declare
@@ -62,13 +65,19 @@ extern int exitstatus;
 extern int64_t bytes;
 
 void printslogan(){
-  pout("smartctl version %s Copyright (C) 2002-4 Bruce Allen\n", PACKAGE_VERSION);
+  pout("smartctl version %s [%s] Copyright (C) 2002-4 Bruce Allen\n", PACKAGE_VERSION, SMARTMONTOOLS_BUILD_HOST);
   pout("Home page is " PACKAGE_HOMEPAGE "\n\n");
   return;
 }
 
-void printcopy(){
+void PrintOneCVS(const char *a_cvs_id){
   char out[CVSMAXLEN];
+  printone(out,a_cvs_id);
+  pout("%s",out);
+  return;
+}
+
+void printcopy(){
   char *configargs=strlen(SMARTMONTOOLS_CONFIGURE_ARGS)?SMARTMONTOOLS_CONFIGURE_ARGS:"[no arguments given]";
 
   pout("smartctl comes with ABSOLUTELY NO WARRANTY. This\n");
@@ -76,32 +85,21 @@ void printcopy(){
   pout("under the terms of the GNU General Public License Version 2.\n");
   pout("See http://www.gnu.org for further details.\n\n");
   pout("CVS version IDs of files used to build this code are:\n");
-  printone(out,atacmdnames_c_cvsid);
-  pout("%s",out);
-  printone(out,atacmds_c_cvsid);
-  pout("%s",out);
-  printone(out,ataprint_c_cvsid);
-  pout("%s",out);
- #if defined(_WIN32) && defined(_MSC_VER)
-  printone(out,int64_vc6_c_cvsid);
-  pout("%s",out);
+  PrintOneCVS(atacmdnames_c_cvsid);
+  PrintOneCVS(atacmds_c_cvsid);
+  PrintOneCVS(ataprint_c_cvsid);
+#if defined(_WIN32) && defined(_MSC_VER)
+  PrintOneCVS(int64_vc6_c_cvsid);
 #endif
- printone(out,knowndrives_c_cvsid);
-  pout("%s",out);
-  printone(out,os_XXXX_c_cvsid);
-  pout("%s",out);
+  PrintOneCVS(knowndrives_c_cvsid);
+  PrintOneCVS(os_XXXX_c_cvsid);
 #ifdef NEED_SOLARIS_ATA_CODE
-  printone(out, os_solaris_ata_s_cvsid);
-  pout("%s",out);
+  PrintOneCVS(os_solaris_ata_s_cvsid);
 #endif
-  printone(out,scsicmds_c_cvsid);
-  pout("%s",out);
-  printone(out,scsiprint_c_cvsid);
-  pout("%s",out);
-  printone(out,smartctl_c_cvsid);
-  pout("%s",out);
-  printone(out,utility_c_cvsid);
-  pout("%s",out);
+  PrintOneCVS(scsicmds_c_cvsid);
+  PrintOneCVS(scsiprint_c_cvsid);
+  PrintOneCVS(smartctl_c_cvsid);
+  PrintOneCVS(utility_c_cvsid);
   pout("\nsmartmontools release " PACKAGE_VERSION " dated " SMARTMONTOOLS_RELEASE_DATE " at " SMARTMONTOOLS_RELEASE_TIME "\n");
   pout("smartmontools build host: " SMARTMONTOOLS_BUILD_HOST "\n");
   pout("smartmontools build configured: " SMARTMONTOOLS_CONFIGURE_DATE "\n");
@@ -144,7 +142,7 @@ void Usage (void){
 "  -q TYPE, --quietmode=TYPE                                           (ATA)\n"
 "         Set smartctl quiet mode to one of: errorsonly, silent\n\n"
 "  -d TYPE, --device=TYPE\n"
-"         Specify device type to one of: ata, scsi, 3ware,N\n\n"
+"         Specify device type to one of: ata, scsi, marvell, 3ware,N\n\n"
 "  -T TYPE, --tolerance=TYPE                                           (ATA)\n"
 "         Tolerance: normal, conservative, permissive, verypermissive\n\n"
 "  -b TYPE, --badsum=TYPE                                              (ATA)\n"
@@ -289,9 +287,6 @@ void printvalidarglistmessage(char opt) {
   return;
 }
 
-
-unsigned char tryata=0,tryscsi=0;
-
 /*      Takes command options and sets features to be run */    
 void ParseOpts (int argc, char** argv){
   int optchar;
@@ -370,13 +365,14 @@ void ParseOpts (int argc, char** argv){
       break;
     case 'd':
       if (!strcmp(optarg,"ata")) {
-        tryata  = TRUE;
-        tryscsi = FALSE;
-        con->escalade = 0;
+	con->controller_type = CONTROLLER_ATA;
+        con->controller_port = 0;
       } else if (!strcmp(optarg,"scsi")) {
-        tryata  = FALSE;
-        tryscsi = TRUE;
-        con->escalade = 0;
+	con->controller_type = CONTROLLER_SCSI;
+        con->controller_port = 0;
+      } else if (!strcmp(optarg,"marvell")) {
+	con->controller_type = CONTROLLER_MARVELL_SATA;
+        con->controller_port = 0;
       } else {
         // look for RAID-type device
         int i;
@@ -396,10 +392,9 @@ void ParseOpts (int argc, char** argv){
           sprintf(extraerror, "Option -d 3ware,N (N=%d) must have 0 <= N <= 15\n", i);
           badarg = TRUE;
         } else {
-          // NOTE: escalade = disk number + 1
-          con->escalade = i+1;
-          tryata  = TRUE;
-          tryscsi = FALSE;
+	  // NOTE: controller_port == disk number + 1
+	  con->controller_type = CONTROLLER_3WARE;
+          con->controller_port = i+1;
         }
         free(s);
       }         
@@ -818,7 +813,6 @@ int main (int argc, char **argv){
   int fd,retval=0;
   char *device;
   smartmonctrl control;
-  int dev_type;
   char *mode=NULL;
 
   // define control block for external functions
@@ -829,26 +823,38 @@ int main (int argc, char **argv){
 
   device = argv[argc-1];
 
-  if (!tryata && !tryscsi) {
-    // user has not specified device type, so guess
-    dev_type = guess_device_type(device);
-    if (GUESS_DEVTYPE_SCSI == dev_type) {
-      tryscsi = 1;
-    } else if (GUESS_DEVTYPE_ATA == dev_type)
-      tryata = 1;
-    else {
-      pout("Smartctl: please specify if this is an ATA or SCSI device with the -d option.\n");
-      UsageSummary();
-      return FAILCMD;
-    }    
+  // If use has specified 3ware controller, determine which interface 
+  if (con->controller_type == CONTROLLER_3WARE) {
+    con->controller_type=guess_device_type(device);
+    if (con->controller_type!=CONTROLLER_3WARE_9000_CHAR && con->controller_type!=CONTROLLER_3WARE_678K_CHAR)
+      con->controller_type = CONTROLLER_3WARE_678K;
+  }
+
+  if (con->controller_type == CONTROLLER_UNKNOWN)
+    con->controller_type=guess_device_type(device);
+  
+  if (con->controller_type == CONTROLLER_UNKNOWN) {
+    pout("Smartctl: please specify device type with the -d option.\n");
+    UsageSummary();
+    return FAILCMD;
   }
   
   // set up mode for open() call.  SCSI case is:
-  mode="SCSI";
-
-  if (tryata)
+  switch (con->controller_type) {
+  case CONTROLLER_SCSI:
+    mode="SCSI";
+    break;
+  case CONTROLLER_3WARE_9000_CHAR:
+    mode="ATA_3WARE_9000";
+    break;
+  case CONTROLLER_3WARE_678K_CHAR:
+    mode="ATA_3WARE_678K";
+    break;
+  default:
     mode="ATA";
-    
+    break;
+  }
+  
   // open device - SCSI devices are opened (O_RDWR | O_NONBLOCK) so the
   // scsi generic device can be used (needs write permission for MODE 
   // SELECT command) plus O_NONBLOCK to stop open hanging if media not
@@ -865,14 +871,19 @@ int main (int argc, char **argv){
   }
 
   // now call appropriate ATA or SCSI routine
-  if (tryata)
-    retval = ataPrintMain(fd);
-  else if (tryscsi)
-    retval = scsiPrintMain(fd);
-  else {
-    pout("Smartctl: specify if this is an ATA or SCSI device with the -d option.\n");
+  switch (con->controller_type) {
+  case CONTROLLER_UNKNOWN:
+    // we should never fall into this branch!
+    pout("Smartctl: please specify device type with the -d option.\n");
     UsageSummary();
-    return FAILCMD;
+    retval = FAILCMD;
+    break;
+  case CONTROLLER_SCSI:
+    retval = scsiPrintMain(fd);
+    break;
+  default:
+    retval = ataPrintMain(fd);
+    break;
   }
   
   return retval;
