@@ -41,7 +41,7 @@
 #include "utility.h"
 
 // Any local header files should be represented by a CVSIDX just below.
-const char* utility_c_cvsid="$Id: utility.c,v 1.51 2004/05/27 16:26:36 card_captor Exp $"
+const char* utility_c_cvsid="$Id: utility.c,v 1.54 2004/08/29 01:08:37 ballen4705 Exp $"
 CONFIG_H_CVSID INT64_H_CVSID UTILITY_H_CVSID;
 
 const char * packet_types[] = {
@@ -316,21 +316,24 @@ void printone(char *block, const char *cvsid){
 
 
 // A replacement for perror() that sends output to our choice of
-// printing.
+// printing. If errno not set then just print message.
 void syserror(const char *message){
-  const char *errormessage;
   
-  // Get the correct system error message:
-  errormessage=strerror(errno);
-
-  // Check that caller has handed a sensible string, and provide
-  // appropriate output. See perrror(3) man page to understand better.
+  if (errno) {
+    // Get the correct system error message:
+    const char *errormessage=strerror(errno);
+    
+    // Check that caller has handed a sensible string, and provide
+    // appropriate output. See perrror(3) man page to understand better.
     if (message && *message)
       pout("%s: %s\n",message, errormessage);
     else
       pout("%s\n",errormessage);
-        
-    return;
+  }
+  else if (message && *message)
+    pout("%s\n",message);
+  
+  return;
 }
 
 // Prints a warning message for a failed regular expression compilation from
@@ -410,6 +413,54 @@ int split_report_arg2(char *s, int *i){
   return 0;
 }
 
+#ifndef HAVE_STRTOULL
+// Replacement for missing strtoull() (Linux with libc < 6, MSVC 6.0)
+// Functionality reduced to split_selective_arg()'s requirements.
+
+static uint64_t strtoull(const char * p, char * * endp, int base)
+{
+  uint64_t result, maxres;
+  int i = 0;
+  char c = p[i++];
+  // assume base == 0
+  if (c == '0') {
+    if (p[i] == 'x' || p[i] == 'X') {
+      base = 16; i++;
+    }
+    else
+      base = 8;
+    c = p[i++];
+  }
+  else
+    base = 10;
+
+  result = 0;
+  maxres = ~(uint64_t)0 / (unsigned)base;
+  for (;;) {
+    unsigned digit;
+    if ('0' <= c && c <= '9')
+      digit = c - '0';
+    else if ('A' <= c && c <= 'Z')
+      digit = c - 'A' + 10;
+    else if ('a' <= c && c <= 'z')
+      digit = c - 'a' + 10;
+    else
+      break;
+    if (digit >= (unsigned)base)
+      break;
+    if (!(   result < maxres
+          || (result == maxres && digit <= ~(uint64_t)0 % (unsigned)base))) {
+      result = ~(uint64_t)0; errno = ERANGE; // return on overflow
+      break;
+    }
+    result = result * (unsigned)base + digit;
+    c = p[i++];
+  }
+  *endp = (char *)p + i - 1;
+  return result;
+}
+#endif // HAVE_STRTOLL
+
 // Splits an argument to the -t option that is assumed to be of the form
 // "selective,%lld-%lld" (prefixes of "0" (for octal) and "0x"/"0X" (for hex)
 // are allowed).  The first long long int is assigned to *start and the second
@@ -496,7 +547,7 @@ char *CustomStrDup(char *ptr, int mustexist, int whatline, const char* file){
   tmp=strdup(ptr);
   
   if (!tmp) {
-    PrintOut(LOG_CRIT, "No memory to duplicate string %s\n", ptr);
+    PrintOut(LOG_CRIT, "No memory to duplicate string %s at line %d of file %s\n", ptr, whatline, file);
     EXIT(EXIT_NOMEM);
   }
   
