@@ -25,7 +25,7 @@
 #ifndef UTILITY_H_
 #define UTILITY_H_
 
-#define UTILITY_H_CVSID "$Id: utility.h,v 1.33 2004/04/14 13:27:20 chrfranke Exp $\n"
+#define UTILITY_H_CVSID "$Id: utility.h,v 1.38 2004/08/18 19:27:36 likewise Exp $\n"
 
 #include <time.h>
 #include <sys/types.h> // for regex.h (according to POSIX)
@@ -77,10 +77,10 @@ int split_report_arg2(char *s, int *i);
 // Function for processing -t selective... option in smartctl
 int split_selective_arg(char *s, uint64_t *start, uint64_t *stop);
 
+
 // Guess device type (ata or scsi) based on device name 
-#define GUESS_DEVTYPE_ATA       0
-#define GUESS_DEVTYPE_SCSI      1
-#define GUESS_DEVTYPE_DONT_KNOW 2
+// Guessing will now use Controller Type defines below
+
 int guess_device_type(const char * dev_name);
 
 // Create and return the list of devices to probe automatically
@@ -157,4 +157,15 @@ void MsecToText(unsigned int msec, char *txt);
 #define PRINT_ON(control)  {if (control->printing_switchable) control->dont_print=0;}
 #define PRINT_OFF(control) {if (control->printing_switchable) control->dont_print=1;}
 
+// possible values for controller_type in extern.h
+#define CONTROLLER_UNKNOWN              0x00
+#define CONTROLLER_ATA                  0x01
+#define CONTROLLER_SCSI                 0x02
+#define CONTROLLER_3WARE                0x03  // set by -d option, but converted to one of three types below
+#define CONTROLLER_3WARE_678K           0x04  // NOT set by guess_device_type()
+#define CONTROLLER_3WARE_9000_CHAR      0x05  // set by guess_device_type()
+#define CONTROLLER_3WARE_678K_CHAR      0x06  // set by guess_device_type()
+#define CONTROLLER_MARVELL_SATA         0x07  // SATA drives behind Marvell controllers
+
+
 #endif
